% This script runs simulated experiments, testing participants who have
% experiential distributions that have different levels of kurtosis, and
% different criteria for reporting that they are confident in categorical 
% perceptual decisions. We compare our new Slope-based test of metacogntive 
% sensitivity to the existing gold-standard approach (as implemented by 
% fit_meta_d_MLE.m | see Maniscalco & Lau (2012) Consciousness & Cognition.
%
% To run the code, you will have to have installed 
% 1) fit_meta_d_MLE.m | see: http://www.columbia.edu/~bsm2105/type2sdt/
% 2) Psignifit | see: https://github.com/wichmann-lab/psignifit/wiki
% 
% Psignifit is used to fit Cumulative Gaussian functions to data. You could 
% edit this code to swap in a more generic Psychometric function fit 


%% Clear memory, and make sure you can target relevant files
clear all % Clear memory
close all % close all figures

% If psignifit has not already been added to your path, do that now
addpath C:\Users\uqdarnol\cloudstor\Toolbox\Psignifit\psignifit-master
% Also, add to the path the directory where fit_meta_d_MLE.m is stored
addpath C:\Users\uqdarnol\cloudstor\Toolbox\Meta_Scripts

%% Setup some generic parameters

% Hard wirded level of confidence noise, to simulate metacognitive
% insensitivity
Confidence_Noise = 0.75;

% Number of trials that will be run per simulated experiment
N_Simulated_Trials = 20000;

% Randomized trial order
Random_Trial_Order = randperm(N_Simulated_Trials);

%% Setup some options that will be used by Psignifit
options = struct;   % initialize as an empty structure, to store curve fitting options
options.sigmoidName = 'norm';    % choose a cumulative Gaussian as the sigmoid
options.expType = "YesNo"        % Specify the design of the 'experiment' - a forced choice (Yes/No) categorisation task 
options.widthalpha     = 0.25;   % Specify value for JND
options.estimateType   = 'mean';
options.stepN=[20,20,10,10,10];

plotOptions1.dataColor = [0,0,0];  % color of the data
plotOptions1.yLabel = 'Pr. Categorisations';
plotOptions1.xLabel = 'Input Values';
plotOptions1.plotAsymptote  = false; 

plotOptions2.dataColor = [0,0,1];  % color of the data
plotOptions2.lineColor = [0,0,1];  % color of the data
plotOptions2.yLabel = 'Pr. Categorisations';
plotOptions2.xLabel = 'Input Values';
plotOptions2.plotAsymptote  = false; 

plotOptions3.dataColor = [1,0,0];  % color of the data
plotOptions3.lineColor = [1,0,0];  % color of the data
plotOptions3.yLabel = 'Pr. Categorisations';
plotOptions3.xLabel = 'Input Values';
plotOptions3.plotAsymptote  = false; 

%% Prepare distributions of experiences for different test values

Ch_SDs = 2;                                      % S.D. of distributions
Category_Criterion = 0;                          % Perceptual category criterion (0 to simulate unbiased observers)
Confidence_Criteria = Ch_SDs*1.25:0.25:Ch_SDs*3; % We will simuluate experiments, where people 
                                                 % have different criteria for having relatively 
                                                 % high confidence in a decision  

% Enter first guess parameters, for lower performance asymptope, PSE, and upper performance asymptope
First_Guesses = [-Ch_SDs*5 0 +Ch_SDs*5];

Test_Range = (First_Guesses(3)-First_Guesses(1))*5; % = 100
Test_Inputs = -Test_Range/2 :0.5:Test_Range/2;      % -50:0.5:+50
N_Test_Levels = size(Test_Inputs,2);                % = 201

Test_Vals = zeros(N_Test_Levels,N_Simulated_Trials/2); % Empty array, that 
                                                       % different distribuitions 
                                                       % will be stored in,
                                                       % to represent the
                                                       % distributions of
                                                       % different
                                                       % experiences that
                                                       % would ensue if we
                                                       % repeatedly sampled
                                                       % different test
                                                       % values

%% Run simulated experiments
Slopes = zeros(size(Confidence_Criteria,2),2);
Slopes(Slopes==0)=NaN;

for N_Kurtosis = 1 : 3
    
    % We cycle through, sampling 3 levels of kurtosis. 3 (a normal
    % distribution), 3.2 and 3.4
    Kurtosis = 3 + ((N_Kurtosis-1)/2)*0.4;
    
    % For each physical test value, calculate a distribution of values that
    % represent the different experiences that would ensue if we presented
    % that physical input 1000 times
    for Sample_N = 1 : N_Test_Levels
        Sample_Mean = -(Test_Range/2) + (((Sample_N-1)/(N_Test_Levels-1))*Test_Range);
        Test_Vals(Sample_N,:) = pearsrnd(Sample_Mean,Ch_SDs,0,Kurtosis,1,N_Simulated_Trials/2);
    end

    %% Prepare staircases to target different regions of the psychological function
    
    % We will run 5 interleaved staircases. Two of these will target
    % extreme values, that should almost always be judged as belonging to
    % opposite categories with high confidence. The middle category will
    % target where the participant switches from predominantly categorising 
    % inputs as belonging to Category A yo predominantly categorising 
    % inputs as belonging to Category B (the PSE). The final two will 
    % target category confidence boundaries. One will target 
    % where the participant switches from predominantly categorising inputs
    % as Category A with high confidence to predominantly categorising 
    % inputs as Category A with low confidence. The other will target where 
    % the participant switches from predominantly categorising inputs as 
    % Category B with low confidence, to predominantly categorising inputs 
    % as Category B with high confidence  
    Staircase_Sample_Pts = [First_Guesses(1) First_Guesses(1)+(First_Guesses(2)-First_Guesses(1))/2 First_Guesses(2) First_Guesses(2)+(First_Guesses(3)-First_Guesses(2))/2 First_Guesses(3)];
    Staircase_Histories = zeros(size(Staircase_Sample_Pts,2),10);
    Staircase_Histories(3:5,:) = 1;

    for Crit_N = 1 : size(Confidence_Criteria,2)
        
        % We keep a count of how many times a nominal physical test value
        % has been tested by a staircase, instigating all the counts at 0.
        % We then use the counts to sample in order one of the 1000 values
        % we have pre-generated for each possible test value
        Tested_Level_Counts = zeros(1,N_Test_Levels);
        
        % Initialise an array to store simulated experimental results in...
        Exp_Results = [];
        for T_num = 1 : N_Simulated_Trials

            % Determine which staircase we will sample on this 'trial'
            if Random_Trial_Order(T_num) < N_Simulated_Trials*0.05
                % Target lower performance asymptope - to ensure performance varies
                % from ~90% Lower categorisations with High Confidence...
                Staircase_N = 1;
            elseif Random_Trial_Order(T_num) >= N_Simulated_Trials*0.05 & Random_Trial_Order(T_num) < N_Simulated_Trials*0.35
                % Target lower confidence boundary
                Staircase_N = 2;
            elseif Random_Trial_Order(T_num) >= N_Simulated_Trials*0.35 & Random_Trial_Order(T_num) < N_Simulated_Trials*0.65
                % Target perceptual category boundary
                Staircase_N = 3;
            elseif Random_Trial_Order(T_num) >= N_Simulated_Trials*0.65 & Random_Trial_Order(T_num) < N_Simulated_Trials*0.95
                % Target upper confidence boundary
                Staircase_N = 4;
            else
                % Target upper performance asymptope - to ensure performance varies
                % upto ~90% Upper categorisations with high confidence
                Staircase_N = 5;
            end
            [row Test_Level] = find(Test_Inputs == Staircase_Sample_Pts(Staircase_N));

            Tested_Level_Counts(Test_Level) = Tested_Level_Counts(Test_Level)+1;
            Test_Val = Test_Vals(Test_Level,Tested_Level_Counts(Test_Level));

            %% Determine trial perceptual category response, according to the perceptual category criterion
            if Test_Val < Category_Criterion
                Category_Response = 1;
            else
                Category_Response = 2;
            end        

            %% Determine clean (no confidence noise) trial confidence response, according to the confidence criterion 
            Clean_Confidence_Resp = abs(Test_Val) / Confidence_Criteria(Crit_N);
            
            %% Determine noisy (added confidence noise) trial confidence response, according to the confidence criterion 
            Noisey_Confidence_Resp = abs(Test_Val) / ( (Confidence_Criteria(Crit_N)-(Confidence_Criteria(Crit_N)*Confidence_Noise)) + (rand(1)*Confidence_Criteria(Crit_N)*Confidence_Noise*2) );
            
            % Limit confidence responses to +1
            if Clean_Confidence_Resp > 1
                Clean_Confidence_Resp = 1;
            end
            if Noisey_Confidence_Resp > 1
                Noisey_Confidence_Resp = 1;
            end
            
            % Record simulated trial data
            Exp_Results(T_num,1) = Test_Inputs(Test_Level); % Physical test
            Exp_Results(T_num,2) = Category_Response-1;     % Perceptual category response 
            Exp_Results(T_num,3) = Clean_Confidence_Resp;   % No confidence noise Confidence Rating
            Exp_Results(T_num,4) = Noisey_Confidence_Resp;  % Comfindence nose Confidence Rating

            %% Update staircases with this trials' perceptual category response...
            Staircase_Histories(Staircase_N,2:10)=Staircase_Histories(Staircase_N,1:9);
            Staircase_Histories(Staircase_N,1)=Category_Response-1;

            % Calculate the mean confidence expressed to this point of
            % the experiment
            Mean_Confidence = mean(Exp_Results(:,3));
            
            % Weight the mean confidence value by the number of trials to date,
            % divided by 20. After 20 trials, we will reply exclusively on
            % the mean expressed level of confidence. Until then, we will
            % accept expressed confidence levels above 0.5 as 'high confidence' 
            % and lesser expressed confidence values as 'low confidence' 
            Mean_Weight = (T_num-1)/20;
            if Mean_Weight > 1
                Mean_Weight=1;
            end
            
            % Sets the confidence value used to ctaegorise confidence.
            % Starts at 0.5, then drifts toward the mean levcel of
            % expressed confidence - solely determined by the mean after 20
            % trials...
            Adj_Confidence_Crit = (Mean_Confidence*Mean_Weight) + (0.5*(1-Mean_Weight));

            % Deifferent update rules for different staorcases
            if Staircase_N == 1 
                % Sample low extreme values. Sample a higher value next
                % trial if the participant has responded Category A on 9/10
                % of the past trials for this staircase & has reported a
                % relatively high level of confidence, otherwise sample a
                % lower value
                if Category_Response == 1 && Clean_Confidence_Resp == 1 && sum(Staircase_Histories(Staircase_N,:)) < 0.1
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)+1;
                else
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)-1;
                end
            elseif Staircase_N == 2
                % Target confidence boundary for the lower perceptual 
                % category. Sample a greater test value next trial if the
                % participant has responded Category A with relatively high
                % confidence. Otherwise, sample a lesser test value next
                % trial
                if Category_Response == 1 & Clean_Confidence_Resp == 1
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)+1;
                else
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)-1;
                end
            elseif Staircase_N == 3
                % Sample perceptual category boundary. Sample a greater 
                % value next trial if the particiapnt has responded 
                % Category A. Otherwise, sample a lesser value next trial.           
                if Category_Response == 1
                    Staircase_Sample_Pts(Staircase_N) =  Staircase_Sample_Pts(Staircase_N)+1;
                else
                    Staircase_Sample_Pts(Staircase_N) =  Staircase_Sample_Pts(Staircase_N)-1;
                end
            elseif Staircase_N == 4
                % Target confidence boundary for the upper perceptual 
                % category. Sample a lesser test value next trial if the
                % participant has responded Category B with relatively high
                % confidence. Otherwise, sample a greater test value next
                % trial
                if Category_Response == 2 & Clean_Confidence_Resp == 1
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)-1;
                else
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)+1;
                end
            else
                % Sample high extreme values. Sample a lower value next
                % trial if the participant has responded Category B on 9/10
                % of the past trials for this staircase & has reported
                % another Category B presentation with relatively high 
                % confidence, otherwise sample a lower value
                if Category_Response == 2 && Clean_Confidence_Resp == 1 && sum(Staircase_Histories(Staircase_N,:)) > 0.9
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)-1;
                else
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)+1;
                end
            end

            % Constrain staircases to the range of possible test values
            if Staircase_Sample_Pts(Staircase_N) < min(Test_Inputs)
                Staircase_Sample_Pts(Staircase_N) = min(Test_Inputs);
            elseif Staircase_Sample_Pts(Staircase_N) > max(Test_Inputs)
                Staircase_Sample_Pts(Staircase_N) = max(Test_Inputs);
            end
        end

        % Convert confidence ratings from values between 0 and 1 to values 
        % of 0 or 1 (i.e. convert these to categorical numbers) 
        Clean_Confidence_Responses = Exp_Results(:,3);
        AVG_Confidence = mean(Clean_Confidence_Responses);
        Clean_Confidence_Responses(Clean_Confidence_Responses<1)=0;
        Clean_Confidence_Responses(Clean_Confidence_Responses>=1)=1;
        Exp_Results(:,3)=Clean_Confidence_Responses;
        
        Store_Confidence_Criteria(Crit_N) = AVG_Confidence;

        % Convert noisy confidence ratings from values between 0 and 1 to  
        % values of 0 or 1 (i.e. convert these to categorical numbers) 
        Noisey_Confidence_Responses = Exp_Results(:,4);
        AVG_Confidence = mean(Noisey_Confidence_Responses);
        Noisey_Confidence_Responses(Noisey_Confidence_Responses<1)=0;
        Noisey_Confidence_Responses(Noisey_Confidence_Responses>=1)=1;
        Exp_Results(:,4)=Noisey_Confidence_Responses;

        % Find all simulated trials that resulted in Perceptual Category A
        % decisions
        [idxs_Low_Cat vals] = find(Exp_Results(:,2) == 0);
        % Find all simulated trials that resulted in Perceptual Category B
        % decisions
        [idxs_High_Cat vals] = find(Exp_Results(:,2) == 1);

        % Find all simulated trials that resulted in relatively low
        % confidence ratings (without any added confidence noise)
        [idxs_Low_Conf_Clean vals] = find(Exp_Results(:,3) == 0);           
        % Find all simulated trials that resulted in relatively high
        % confidence ratings (without any added confidence noise)
        [idxs_High_Conf_Clean vals] = find(Exp_Results(:,3) == 1);  

        % Find all simulated trials that resulted in relatively low
        % confidence ratings (with added confidence noise)
        [idxs_Low_Conf_Noisey vals] = find(Exp_Results(:,4) == 0);   
        % Find all simulated trials that resulted in relatively high
        % confidence ratings (without any added confidence noise)
        [idxs_High_Conf_Noisey vals] = find(Exp_Results(:,4) == 1);   

        % Create array with all tested values
        Tested_Vals = unique(Exp_Results(:,1));
        Results_Array_Perception = [];

        Results_Array_Confidence_Low_Category_Clean = [];
        Results_Array_Confidence_High_Category_Clean = [];

        Results_Array_Confidence_Low_Category_Noisey = [];
        Results_Array_Confidence_High_Category_Noisey = [];

        for row = 1 : size(Tested_Vals,1)
            [idxs vals] = find(Exp_Results(:,1) == Tested_Vals(row));
            Results_Array_Perception(row,1)=Tested_Vals(row);
            Results_Array_Perception(row,2)=sum(Exp_Results(idxs,2));
            Results_Array_Perception(row,3)=size(idxs,1);

            [hmmm1 bop1] = intersect(idxs,idxs_Low_Cat);    % Tests categorised LOW
            [hmmm2 bop2] = intersect(idxs,idxs_High_Conf_Clean);  % Test resulting in High Confidence - No confidence noise
            [hmmm3 bop3] = intersect(hmmm1,hmmm2);          % Tests BOTH categorised LOW & High Confidence
            Results_Array_Confidence_Low_Category_Clean(row,1)=Tested_Vals(row);
            if size(hmmm1,1) == 0
                % No LOW categorisations - so set to full number of tests
                Results_Array_Confidence_Low_Category_Clean(row,2)=size(idxs,1) ;
            elseif size(hmmm3,1) == 0
                % No LOW categorisations w/ High Confidence - so set to full number of tests
                Results_Array_Confidence_Low_Category_Clean(row,2)=size(idxs,1) ;
            else
                Results_Array_Confidence_Low_Category_Clean(row,2)=size(idxs,1) - sum(Exp_Results(hmmm3,3));
            end
            Results_Array_Confidence_Low_Category_Clean(row,3)=size(idxs,1);


            [hmmm2 bop2] = intersect(idxs,idxs_High_Conf_Noisey);    % Test resulting in High Confidence - No confidence noise
            [hmmm3 bop3] = intersect(hmmm1,hmmm2);                  % Tests BOTH categorised LOW & High Confidence
            Results_Array_Confidence_Low_Category_Noisey(row,1)=Tested_Vals(row);
            if size(hmmm1,1) == 0
                % No LOW categorisations - so set to full number of tests
                Results_Array_Confidence_Low_Category_Noisey(row,2)=size(idxs,1) ;
            elseif size(hmmm3,1) == 0
                % No LOW categorisations w/ High Confidence - so set to full number of tests
                Results_Array_Confidence_Low_Category_Noisey(row,2)=size(idxs,1) ;
            else
                Results_Array_Confidence_Low_Category_Noisey(row,2)=size(idxs,1) - sum(Exp_Results(hmmm3,4));
            end
            Results_Array_Confidence_Low_Category_Noisey(row,3)=size(idxs,1);


            [hmmm1 bop1] = intersect(idxs,idxs_High_Cat);
            [hmmm2 bop2] = intersect(idxs,idxs_High_Conf_Clean);
            [hmmm3 bop3] = intersect(hmmm1,hmmm2);
            Results_Array_Confidence_High_Category_Clean(row,1)=Tested_Vals(row);
            if size(hmmm3,1) == 0
                Results_Array_Confidence_High_Category_Clean(row,2)=0;
            else
                Results_Array_Confidence_High_Category_Clean(row,2)=sum(Exp_Results(hmmm3,3));
            end
            Results_Array_Confidence_High_Category_Clean(row,3)=size(idxs,1);

            [hmmm2 bop2] = intersect(idxs,idxs_High_Conf_Noisey);
            [hmmm3 bop3] = intersect(hmmm1,hmmm2);
            Results_Array_Confidence_High_Category_Noisey(row,1)=Tested_Vals(row);
            if size(hmmm3,1) == 0
                Results_Array_Confidence_High_Category_Noisey(row,2)=0;
            else
                Results_Array_Confidence_High_Category_Noisey(row,2)=sum(Exp_Results(hmmm3,4));
            end
            Results_Array_Confidence_High_Category_Noisey(row,3)=size(idxs,1);

        end

        options.expType = "YesNo"        % Specify the design of the 'experiment' - a forced choice categorisation task 

        % Fit a cumulative Gaussian psychmetric curve to categorisation data
        Psychometric = psignifit(Results_Array_Perception,options);
        figure(1)
        clf
        h1 = plotPsych(Psychometric,plotOptions1);
        PSE_Perception = Psychometric.Fit(1);
        [val PSE_idx]= find(h1.XData >= PSE_Perception,1);
        PSE_Pr = h1.YData(PSE_idx);
        Slopes(Crit_N,1) = Psychometric.Fit(2);

        Psychometric2 = psignifit(Results_Array_Confidence_Low_Category_Clean,options);
        figure(1)
        hold on
        h2 = plotPsych(Psychometric2,plotOptions2);
        PSE = Psychometric2.Fit(1);
        [val PSE_idx]= find(h2.XData >= PSE,1);
        PSE_Pr = h2.YData(PSE_idx);
        Lower_Cat_Confidence_Slope = Psychometric2.Fit(2);


        Psychometric3 = psignifit(Results_Array_Confidence_High_Category_Clean,options);
        figure(1)
        hold on
        h3 = plotPsych(Psychometric3,plotOptions3);
        PSE = Psychometric3.Fit(1);
        [val PSE_idx]= find(h3.XData >= PSE,1);
        PSE_Pr = h3.YData(PSE_idx);
        Upper_Cat_Confidence_Slope = Psychometric3.Fit(2); 

        Candidate_Slopes = abs(([Slopes(Crit_N,1)-Lower_Cat_Confidence_Slope Slopes(Crit_N,1)-Upper_Cat_Confidence_Slope]));
        [Biggest_Diff idx] = max(Candidate_Slopes);
        if idx == 1
            Slopes(Crit_N,2) = Lower_Cat_Confidence_Slope;
        else
            Slopes(Crit_N,2) = Upper_Cat_Confidence_Slope;
        end

        Psychometric2 = psignifit(Results_Array_Confidence_Low_Category_Noisey,options);
        figure(3)
        clf
        h1 = plotPsych(Psychometric,plotOptions1);
        hold on
        
        h2 = plotPsych(Psychometric2,plotOptions2);
        PSE = Psychometric2.Fit(1);
        [val PSE_idx]= find(h2.XData >= PSE,1);
        PSE_Pr = h2.YData(PSE_idx);
        Lower_Cat_Confidence_Slope = Psychometric2.Fit(2); 


        Psychometric3 = psignifit(Results_Array_Confidence_High_Category_Noisey,options);
        figure(3)
        hold on
        h3 = plotPsych(Psychometric3,plotOptions3);
        PSE = Psychometric3.Fit(1);
        [val PSE_idx]= find(h3.XData >= PSE,1);
        PSE_Pr = h3.YData(PSE_idx);
        Upper_Cat_Confidence_Slope = Psychometric3.Fit(2);

        Candidate_Slopes = abs(([Slopes(Crit_N,1)-Lower_Cat_Confidence_Slope Slopes(Crit_N,1)-Upper_Cat_Confidence_Slope]));
        [Biggest_Diff idx] = max(Candidate_Slopes);
        if idx == 1
            Slopes(Crit_N,3) = Lower_Cat_Confidence_Slope;
        else
            Slopes(Crit_N,3) = Upper_Cat_Confidence_Slope;
        end

        %%

        % Create pseudo random distributions of Signal and Noise values
        Noise_Dist = pearsrnd(-Ch_SDs*0.5,Ch_SDs,0,Kurtosis,1,N_Simulated_Trials/2);
        Signal_Dist = pearsrnd(+Ch_SDs*0.5,Ch_SDs,0,Kurtosis,1,N_Simulated_Trials/2);

        % Convert Signal and Noise values into (unsigned) absolute values for
        % confidence classifications
        Noise_Confidence_Responses = abs(Noise_Dist);
        Signal_Confidence_Responses = abs(Signal_Dist);

        % Find Noise values to classify as having elicited high- or
        % low-confidence
        [vals idxs_Noise_High_Confidence] = find(Noise_Confidence_Responses >= Confidence_Criteria(Crit_N));
        [vals idxs_Noise_Low_Confidence] = find(Noise_Confidence_Responses < Confidence_Criteria(Crit_N));
        % Find Noise values to categorise as a Noise (values below category 
        % criterion) or signal (values equal to or above category criterion)
        [vals idxs_Noise_Cat1] = find(Noise_Dist < Category_Criterion);
        [vals idxs_Noise_Cat2] = find(Noise_Dist >= Category_Criterion);

        % Find Noise values classified as Noise with high-confidence
        [idxs_High_Conf_Noise_N vals]=intersect(idxs_Noise_High_Confidence,idxs_Noise_Cat1);
        % Find Noise values classified as Noise with low-confidence
        [idxs_Low_Conf_Noise_N vals]=intersect(idxs_Noise_Low_Confidence,idxs_Noise_Cat1); 
        % Find Noise values classified as Signal with low-confidence
        [idxs_Low_Conf_Signal_N vals]=intersect(idxs_Noise_Low_Confidence,idxs_Noise_Cat2); 
        % Find Noise values classified as Signal with high-confidence
        [idxs_High_Conf_Signal_N vals]=intersect(idxs_Noise_High_Confidence,idxs_Noise_Cat2); 


        % Find Signal values to classify as having elicited high- or
        % low-confidence
        [vals idxs_Signal_High_Confidence] = find(Signal_Confidence_Responses >= Confidence_Criteria(Crit_N));
        [vals idxs_Signal_Low_Confidence] = find(Signal_Confidence_Responses < Confidence_Criteria(Crit_N));
        % Find Signal values to categorise as a Noise (values below category 
        % criterion) or signal (values equal to or above category criterion)
        [vals idxs_Signal_Cat1] = find(Signal_Dist < Category_Criterion);
        [vals idxs_Signal_Cat2] = find(Signal_Dist >= Category_Criterion);

        % Find Signal values classified as Noise with high-confidence
        [idxs_High_Conf_Noise_S vals]=intersect(idxs_Signal_High_Confidence,idxs_Signal_Cat1);
        % Find Signal values classified as Noise with low-confidence
        [idxs_Low_Conf_Noise_S vals]=intersect(idxs_Signal_Low_Confidence,idxs_Signal_Cat1); 
        % Find Signal values classified as Signal with low-confidence
        [idxs_Low_Conf_Signal_S vals]=intersect(idxs_Signal_Low_Confidence,idxs_Signal_Cat2); 
        % Find Signal values classified as Signal with high-confidence
        [idxs_High_Conf_Signal_S vals]=intersect(idxs_Signal_High_Confidence,idxs_Signal_Cat2); 

        % Create meta-d array, and fill the first row with Noise
        % classifications X Confidence
        Meta_D_Array=[];
        Meta_D_Array(1,1) = size(idxs_High_Conf_Noise_N,2);
        Meta_D_Array(1,2) = size(idxs_Low_Conf_Noise_N,2);
        Meta_D_Array(1,3) = size(idxs_Low_Conf_Signal_N,2);
        Meta_D_Array(1,4) = size(idxs_High_Conf_Signal_N,2);

        % Fill second row with Signal classifications X Confidence
        Meta_D_Array(2,1) = size(idxs_High_Conf_Noise_S,2);
        Meta_D_Array(2,2) = size(idxs_Low_Conf_Noise_S,2);
        Meta_D_Array(2,3) = size(idxs_Low_Conf_Signal_S,2);
        Meta_D_Array(2,4) = size(idxs_High_Conf_Signal_S,2);


        if min(min(Meta_D_Array)) > 0
            % Conduct SDT-based analysis of confidence
            Analysis = fit_meta_d_MLE(Meta_D_Array(1,:),Meta_D_Array(2,:));
            % Store estimated d' values for excess kurtosis simmulations
            dPrimes(Crit_N)=Analysis.d1;
            % Store estimated meta d' values for excess kurtosis simmulations
            Meta_dPrimes(Crit_N)=Analysis.meta_d
        else
            % Store estimated d' values for excess kurtosis simmulations
            dPrimes(Crit_N)=NaN;
            % Store estimated meta d' values for excess kurtosis simmulations
            Meta_dPrimes(Crit_N)=NaN;
        end        

        %%
        idxs_Noisey_High_Confidence=[];
        idxs_Noisey_Low_Confidence=[];
        idxs_Signaly_High_Confidence=[];
        idxs_Signaly_Low_Confidence=[];

        Noisey_Confidence_Crit = (Confidence_Criteria(Crit_N)-(Confidence_Criteria(Crit_N)*Confidence_Noise)) + rand(2,size(Noise_Dist,2))*(Confidence_Criteria(Crit_N)*Confidence_Noise*2);
        for t = 1 : size(Noisey_Confidence_Crit,2)

            if Noise_Confidence_Responses(t) >= Noisey_Confidence_Crit(1,t)
                idxs_Noisey_High_Confidence(end+1) = t;
            else
                idxs_Noisey_Low_Confidence(end+1) = t;
            end

            if Signal_Confidence_Responses(t) >= Noisey_Confidence_Crit(2,t)
                idxs_Signaly_High_Confidence(end+1) = t;
            else
                idxs_Signaly_Low_Confidence(end+1) = t;
            end

        end
        % Find Noise values classified as Noise with high-confidence
        [idxs_High_Conf_Noisey_N vals]=intersect(idxs_Noisey_High_Confidence,idxs_Noise_Cat1);
        % Find Noise values classified as Noise with low-confidence
        [idxs_Low_Conf_Noisey_N vals]=intersect(idxs_Noisey_Low_Confidence,idxs_Noise_Cat1); 
        % Find Noise values classified as Signal with low-confidence
        [idxs_Low_Conf_Signaly_N vals]=intersect(idxs_Noisey_Low_Confidence,idxs_Noise_Cat2); 
        % Find Noise values classified as Signal with high-confidence
        [idxs_High_Conf_Signaly_N vals]=intersect(idxs_Noisey_High_Confidence,idxs_Noise_Cat2); 


        % Find Signal values classified as Noise with high-confidence
        [idxs_High_Conf_Noisey_S vals]=intersect(idxs_Signaly_High_Confidence,idxs_Signal_Cat1);
        % Find Signal values classified as Noise with low-confidence
        [idxs_Low_Conf_Noisey_S vals]=intersect(idxs_Signaly_Low_Confidence,idxs_Signal_Cat1); 
        % Find Signal values classified as Signal with low-confidence
        [idxs_Low_Conf_Signaly_S vals]=intersect(idxs_Signaly_Low_Confidence,idxs_Signal_Cat2); 
        % Find Signal values classified as Signal with high-confidence
        [idxs_High_Conf_Signaly_S vals]=intersect(idxs_Signaly_High_Confidence,idxs_Signal_Cat2); 

        % Create meta-d array, and fill the first row with Noise
        % classifications X Confidence
        Meta_D_Array=[];
        Meta_D_Array(1,1) = size(idxs_High_Conf_Noisey_N,2);
        Meta_D_Array(1,2) = size(idxs_Low_Conf_Noisey_N,2);
        Meta_D_Array(1,3) = size(idxs_Low_Conf_Signaly_N,2);
        Meta_D_Array(1,4) = size(idxs_High_Conf_Signaly_N,2);

        % Fill second row with Signal classifications X Confidence
        Meta_D_Array(2,1) = size(idxs_High_Conf_Noisey_S,2);
        Meta_D_Array(2,2) = size(idxs_Low_Conf_Noisey_S,2);
        Meta_D_Array(2,3) = size(idxs_Low_Conf_Signaly_S,2);
        Meta_D_Array(2,4) = size(idxs_High_Conf_Signaly_S,2);

        if min(min(Meta_D_Array)) > 0
            % Conduct SDT-based analysis of confidence
            Analysis = fit_meta_d_MLE(Meta_D_Array(1,:),Meta_D_Array(2,:));
            % Store estimated d' values for excess kurtosis simmulations
            Noisey_dPrimes(Crit_N)=Analysis.d1;
            % Store estimated meta d' values for excess kurtosis simmulations
            Noisey_Meta_dPrimes(Crit_N)=Analysis.meta_d
        else
            % Store estimated d' values for excess kurtosis simmulations
            Noisey_dPrimes(Crit_N)=NaN;
            % Store estimated meta d' values for excess kurtosis simmulations
            Noisey_Meta_dPrimes(Crit_N)=NaN;
        end


    end

    Slope_Ratios_Clean = Slopes(:,1)./Slopes(:,2);
    Slope_Ratios_Noisey = Slopes(:,1)./Slopes(:,3);

    Meta_d_Ratios = Meta_dPrimes./dPrimes;
    Grey_Metas = isnan(Meta_d_Ratios);
    Grey_Metas = double(Grey_Metas);
    Grey_Metas(Grey_Metas==0)=NaN;

    Noisey_Meta_d_Ratios = Noisey_Meta_dPrimes./Noisey_dPrimes;
    Grey_Noisey_Metas = isnan(Noisey_Meta_d_Ratios);
    Grey_Noisey_Metas = double(Grey_Noisey_Metas);
    Grey_Noisey_Metas(Grey_Noisey_Metas==0)=NaN;

    The_Mins = [min(Slope_Ratios_Clean) min(Slope_Ratios_Noisey) min(Meta_d_Ratios) min(Noisey_Meta_d_Ratios)];
    The_Min = min(The_Mins);

    The_Maxs = [max(Slope_Ratios_Clean) max(Slope_Ratios_Noisey) max(Meta_d_Ratios) max(Noisey_Meta_d_Ratios)];
    The_Max = max(The_Maxs);

    figure(4)

    subplot(2,2,1)
    hold on
    if N_Kurtosis == 1
        h=fill([Confidence_Criteria(1),Confidence_Criteria(1),max(Confidence_Criteria),max(Confidence_Criteria)],[0.95,1.05,1.05,0.95],'g','LineStyle','none');
        h.FaceAlpha=0.25;
        plot([Confidence_Criteria(1),max(Confidence_Criteria)],[1,1],'-g','LineWidth',3)
        scatter(Confidence_Criteria,Slope_Ratios_Clean,'k','filled')
    elseif N_Kurtosis == 2
        scatter(Confidence_Criteria,Slope_Ratios_Clean,'b','filled')
    else
        scatter(Confidence_Criteria,Slope_Ratios_Clean,'r','filled')
    end
    xlabel('Confidence Criteria (degs)','FontSize',16,'FontWeight','bold')  
    ylabel('Slope Ratios','FontSize',16,'FontWeight','bold')  
    box off
    ax=gca;
    ax.FontSize = 14; 
    ylim([The_Min The_Max])
    xlim([Confidence_Criteria(1) max(Confidence_Criteria)])
    title({'Slope Tests' 'No Confidence Noise'},'FontSize',16,'FontWeight','bold')


    subplot(2,2,2)
    hold on
    if N_Kurtosis == 1
        plot([Confidence_Criteria(1),max(Confidence_Criteria)],[1,1],'-g','LineWidth',5)
        h=fill([Confidence_Criteria(1),Confidence_Criteria(1),max(Confidence_Criteria),max(Confidence_Criteria)],[0.95,1.05,1.05,0.95],'g','LineStyle','none');
        h.FaceAlpha=0.25;
        hold on
        plot([Confidence_Criteria(1),max(Confidence_Criteria)],[1,1],'-g','LineWidth',3)
        scatter(Confidence_Criteria,Slope_Ratios_Noisey,'k','filled')
    elseif N_Kurtosis == 2
        scatter(Confidence_Criteria,Slope_Ratios_Noisey,'b','filled')
    else
        scatter(Confidence_Criteria,Slope_Ratios_Noisey,'r','filled')
    end
    xlabel('Confidence Criteria (degs)','FontSize',16,'FontWeight','bold')  
    ylabel('Slope Ratios','FontSize',16,'FontWeight','bold')  
    box off
    ax=gca;
    ax.FontSize = 14; 
    ylim([The_Min The_Max])
    xlim([Confidence_Criteria(1) max(Confidence_Criteria)])
    title({'Slope Tests' '75% Confidence Noise'},'FontSize',16,'FontWeight','bold')


    subplot(2,2,3)
    hold on
    if N_Kurtosis == 1
        h=fill([Confidence_Criteria(1),Confidence_Criteria(1),max(Confidence_Criteria),max(Confidence_Criteria)],[0.95,1.05,1.05,0.95],'g','LineStyle','none');
        h.FaceAlpha=0.25;
        hold on
        plot([Confidence_Criteria(1),max(Confidence_Criteria)],[1,1],'-g','LineWidth',3)
        scatter(Confidence_Criteria,Meta_d_Ratios,'k','filled')
    elseif N_Kurtosis == 2
        scatter(Confidence_Criteria,Meta_d_Ratios,'b','filled')
    else
        scatter(Confidence_Criteria,Meta_d_Ratios,'r','filled')
    end
    scatter(Confidence_Criteria,Grey_Metas,'c','filled')
    xlabel('Confidence Criteria (degs)','FontSize',16,'FontWeight','bold')  
    ylabel('Meta dPrime : dPrime Ratios','FontSize',16,'FontWeight','bold')  
    box off
    ax=gca;
    ax.FontSize = 14; 
    ylim([The_Min The_Max])
    xlim([Confidence_Criteria(1) max(Confidence_Criteria)])
    title({'meta dPrime : dPrime' 'No Confidence Noise'},'FontSize',16,'FontWeight','bold')


    subplot(2,2,4)
    hold on
    if N_Kurtosis == 1
        plot([Confidence_Criteria(1),max(Confidence_Criteria)],[1,1],'-g','LineWidth',5)
        h=fill([Confidence_Criteria(1),Confidence_Criteria(1),max(Confidence_Criteria),max(Confidence_Criteria)],[0.95,1.05,1.05,0.95],'g','LineStyle','none');
        h.FaceAlpha=0.25;
        hold on
        plot([Confidence_Criteria(1),max(Confidence_Criteria)],[1,1],'-g','LineWidth',3)
        scatter(Confidence_Criteria,Noisey_Meta_d_Ratios,'k','filled')
    elseif N_Kurtosis == 2
        scatter(Confidence_Criteria,Noisey_Meta_d_Ratios,'b','filled')
    else
        scatter(Confidence_Criteria,Noisey_Meta_d_Ratios,'r','filled')
    end
    scatter(Confidence_Criteria,Grey_Noisey_Metas,'c','filled')
    xlabel('Confidence Criteria (degs)','FontSize',16,'FontWeight','bold')  
    ylabel('Meta dPrime : dPrime Ratios','FontSize',16,'FontWeight','bold')  
    box off
    ax=gca;
    ax.FontSize = 14; 
    ylim([The_Min The_Max])
    xlim([Confidence_Criteria(1) max(Confidence_Criteria)])
    title({'Meta dPrime : dPrime' '75% Confidence Noise'},'FontSize',16,'FontWeight','bold')
end
