% This script runs simulated experiments, testing participants who have
% experiential distributions that have different levels of opposite skew, 
% and different criteria for reporting that they are confident in  
% categorical perceptual decisions. We compare our new Slope-based test of  
% metacogntive sensitivity to the existing gold-standard approach (as 
% implemented by fit_meta_d_MLE.m | see Maniscalco & Lau (2012) 
% Consciousness & Cognition.
%
% To run the code, you will have to have installed 
% 1) fit_meta_d_MLE.m | see: http://www.columbia.edu/~bsm2105/type2sdt/
% 2) Psignifit | see: https://github.com/wichmann-lab/psignifit/wiki
% 
% Psignifit is used to fit Cumulative Gaussian functions to data. You could 
% edit this code to swap in a more generic Psychometric function fit 

%% Clear memory, and make sure you can target relevant files
clear all % Clear memory
close all % close all figures

% If psignifit has not already been added to your path, do that now
addpath C:\Users\uqdarnol\cloudstor\Toolbox\Psignifit\psignifit-master
% Also, add to the path the directory where fit_meta_d_MLE.m is stored
addpath C:\Users\uqdarnol\cloudstor\Toolbox\Meta_Scripts


%% Setup some generic parameters

Confidence_Noise = 0.75;

N_Simulated_Trials = 20000;

Random_Trial_Order = randperm(N_Simulated_Trials);

%% Setup some options that will be used by Psignifit
options = struct;   % initialize as an empty structure, to store curve fitting options
options.sigmoidName = 'norm';    % choose a cumulative Gaussian as the sigmoid
options.expType = "YesNo"        % Specify the design of the 'experiment' - a forced choice categorisation task 
options.estimateType   = 'mean';
options.stepN=[20,20,10,10,10];

plotOptions.dataColor = [0,0,0];  % color of the data
plotOptions.yLabel = 'Pr. Right Categorisations';
plotOptions.xLabel = 'Input Directions';

%% Prepare distributions of experiences for different test values
Kurtosis = 3;
Ch_SDs = 2;

Category_Criterion = 0;
Confidence_Criteria = Ch_SDs*1.25:0.25:Ch_SDs*3;

% Enter first guess parameters, for lower performance asymptope, PSE, and
% upper performance asymptope
First_Guesses = [-Ch_SDs*5 0 +Ch_SDs*5];

Test_Range = (First_Guesses(3)-First_Guesses(1))*5;
Test_Inputs = -Test_Range/2 :0.5:Test_Range/2; 
N_Test_Levels = size(Test_Inputs,2);

Test_Vals = zeros(N_Test_Levels,N_Simulated_Trials/2); % Empty array, that 
                                                       % different distribuitions 
                                                       % will be stored in,
                                                       % to represent the
                                                       % distributions of
                                                       % different
                                                       % experiences that
                                                       % would ensue if we
                                                       % repeatedly sampled
                                                       % different test
                                                       % values


%% Run simulated experiments
Slopes = zeros(size(Confidence_Criteria,2),2);
Slopes(Slopes==0)=NaN;

for N_Skew = 1 : 3
    
    Kurtosis = 3;
    Skew = ((N_Skew-1)/2)*0.16;
    
    for Sample_N = 1 : N_Test_Levels
        Sample_Mean = -(Test_Range/2) + (((Sample_N-1)/(N_Test_Levels-1))*Test_Range);
        if Sample_Mean < 0
            Test_Vals(Sample_N,:) = pearsrnd(Sample_Mean,Ch_SDs,+Skew,Kurtosis,1,N_Simulated_Trials/2);
        else
            Test_Vals(Sample_N,:) = pearsrnd(Sample_Mean,Ch_SDs,-Skew,Kurtosis,1,N_Simulated_Trials/2);
        end
    end

    %% Prepare staircases to target different regions of the psychological function

    Staircase_Sample_Pts = [First_Guesses(1) First_Guesses(1)+(First_Guesses(2)-First_Guesses(1))/2 First_Guesses(2) First_Guesses(2)+(First_Guesses(3)-First_Guesses(2))/2 First_Guesses(3)];
    Staircase_Histories = zeros(size(Staircase_Sample_Pts,2),10);
    Staircase_Histories(3:5,:) = 1;

    for Crit_N = 1 : size(Confidence_Criteria,2)

        Tested_Level_Counts = zeros(1,N_Test_Levels);
        Exp_Results = [];
        for T_num = 1 : N_Simulated_Trials

            if Random_Trial_Order(T_num) < N_Simulated_Trials*0.05
                % Target lower performance asymptope - to ensure performance varies
                % from 100% Lower categorisations...
                Staircase_N = 1;
            elseif Random_Trial_Order(T_num) >= N_Simulated_Trials*0.05 & Random_Trial_Order(T_num) < N_Simulated_Trials*0.35
                % Target lower confidence boundary
                Staircase_N = 2;
            elseif Random_Trial_Order(T_num) >= N_Simulated_Trials*0.35 & Random_Trial_Order(T_num) < N_Simulated_Trials*0.65
                % Target perceptual category boundary
                Staircase_N = 3;
            elseif Random_Trial_Order(T_num) >= N_Simulated_Trials*0.65 & Random_Trial_Order(T_num) < N_Simulated_Trials*0.95
                % Target upper confidence boundary
                Staircase_N = 4;
            else
                % Target upper performance asymptope - to ensure performance varies
                % upto 100% Upper categorisations
                Staircase_N = 5;
            end
            [row Test_Level] = find(Test_Inputs == Staircase_Sample_Pts(Staircase_N));

            Tested_Level_Counts(Test_Level) = Tested_Level_Counts(Test_Level)+1;
            Test_Val = Test_Vals(Test_Level,Tested_Level_Counts(Test_Level));

            %% Determine trial responses, according to perceptual category and confidence criteria
            if Test_Val < Category_Criterion
                Category_Response = 1;
            else
                Category_Response = 2;
            end        

            Clean_Confidence_Resp = abs(Test_Val) / Confidence_Criteria(Crit_N);
            Noisey_Confidence_Resp = abs(Test_Val) / ( (Confidence_Criteria(Crit_N)-(Confidence_Criteria(Crit_N)*Confidence_Noise)) + (rand(1)*Confidence_Criteria(Crit_N)*Confidence_Noise*2) );

            if Clean_Confidence_Resp > 1
                Clean_Confidence_Resp = 1;
            end
            if Noisey_Confidence_Resp > 1
                Noisey_Confidence_Resp = 1;
            end

            Exp_Results(T_num,1) = Test_Inputs(Test_Level);
            Exp_Results(T_num,2) = Category_Response-1;
            Exp_Results(T_num,3) = Clean_Confidence_Resp;
            Exp_Results(T_num,4) = Noisey_Confidence_Resp;

            %% Adjust staircases...
            Staircase_Histories(Staircase_N,2:10)=Staircase_Histories(Staircase_N,1:9);
            Staircase_Histories(Staircase_N,1)=Category_Response-1;

            Median_Confidence = mean(Exp_Results(:,3));
            Median_Weight = (T_num-1)/20;
            if Median_Weight > 1
                Median_Weight=1;
            end
            Adj_Confidence_Crit = (Median_Confidence*Median_Weight) + (0.5*(1-Median_Weight));

            if Staircase_N == 1 
                % Sample extremeties for lower category
                if Category_Response == 1 & sum(Staircase_Histories(Staircase_N,:)) < 0.1
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)+1;
                else
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)-1;
                end
            elseif Staircase_N == 2
                % Sample high-confidence boundary for lower category
                if Category_Response == 1 & Clean_Confidence_Resp == 1
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)+1;
                else
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)-1;
                end
            elseif Staircase_N == 3
                % Sample perceptual category boundary             
                if Category_Response == 1
                    Staircase_Sample_Pts(Staircase_N) =  Staircase_Sample_Pts(Staircase_N)+1;
                else
                    Staircase_Sample_Pts(Staircase_N) =  Staircase_Sample_Pts(Staircase_N)-1;
                end
            elseif Staircase_N == 4
                % Sample high-confidence boundary for upper category            
                if Category_Response == 2 & Clean_Confidence_Resp == 1
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)-1;
                else
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)+1;
                end
            else
                % Sample extremeties for upper category
                if Category_Response == 2 & sum(Staircase_Histories(Staircase_N,:)) > 0.9
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)-1;
                else
                    Staircase_Sample_Pts(Staircase_N) = Staircase_Sample_Pts(Staircase_N)+1;
                end
            end

            % Constrain staircases to test range
            if Staircase_Sample_Pts(Staircase_N) < min(Test_Inputs)
                Staircase_Sample_Pts(Staircase_N) = min(Test_Inputs);
            elseif Staircase_Sample_Pts(Staircase_N) > max(Test_Inputs)
                Staircase_Sample_Pts(Staircase_N) = max(Test_Inputs);
            end
        end

        Clean_Confidence_Responses = Exp_Results(:,3);
        AVG_Confidence = mean(Clean_Confidence_Responses);
        Clean_Confidence_Responses(Clean_Confidence_Responses<1)=0;
        Clean_Confidence_Responses(Clean_Confidence_Responses>=1)=1;
        Exp_Results(:,3)=Clean_Confidence_Responses;

        Noisey_Confidence_Responses = Exp_Results(:,4);
        AVG_Confidence = mean(Noisey_Confidence_Responses);
        Noisey_Confidence_Responses(Noisey_Confidence_Responses<1)=0;
        Noisey_Confidence_Responses(Noisey_Confidence_Responses>=1)=1;
        Exp_Results(:,4)=Noisey_Confidence_Responses;

        [idxs_Low_Cat vals] = find(Exp_Results(:,2) == 0);
        [idxs_High_Cat vals] = find(Exp_Results(:,2) == 1);

        [idxs_Low_Conf_Clean vals] = find(Exp_Results(:,3) == 0);   
        [idxs_High_Conf_Clean vals] = find(Exp_Results(:,3) == 1);  

        [idxs_Low_Conf_Noisey vals] = find(Exp_Results(:,4) == 0);   
        [idxs_High_Conf_Noisey vals] = find(Exp_Results(:,4) == 1);   

        Tested_Vals = unique(Exp_Results(:,1));
        Results_Array_Perception = [];

        Results_Array_Confidence_Low_Category_Clean = [];
        Results_Array_Confidence_High_Category_Clean = [];

        Results_Array_Confidence_Low_Category_Noisey = [];
        Results_Array_Confidence_High_Category_Noisey = [];

        for row = 1 : size(Tested_Vals,1)
            [idxs vals] = find(Exp_Results(:,1) == Tested_Vals(row));
            Results_Array_Perception(row,1)=Tested_Vals(row);
            Results_Array_Perception(row,2)=sum(Exp_Results(idxs,2));
            Results_Array_Perception(row,3)=size(idxs,1);

            [hmmm1 bop1] = intersect(idxs,idxs_Low_Cat);    % Tests categorised LOW
            [hmmm2 bop2] = intersect(idxs,idxs_High_Conf_Clean);  % Test resulting in High Confidence - No confidence noise
            [hmmm3 bop3] = intersect(hmmm1,hmmm2);          % Tests BOTH categorised LOW & High Confidence
            Results_Array_Confidence_Low_Category_Clean(row,1)=Tested_Vals(row);
            if size(hmmm1,1) == 0
                % No LOW categorisations - so set to full number of tests
                Results_Array_Confidence_Low_Category_Clean(row,2)=size(idxs,1) ;
            elseif size(hmmm3,1) == 0
                % No LOW categorisations w/ High Confidence - so set to full number of tests
                Results_Array_Confidence_Low_Category_Clean(row,2)=size(idxs,1) ;
            else
                Results_Array_Confidence_Low_Category_Clean(row,2)=size(idxs,1) - sum(Exp_Results(hmmm3,3));
            end
            Results_Array_Confidence_Low_Category_Clean(row,3)=size(idxs,1);


            [hmmm2 bop2] = intersect(idxs,idxs_High_Conf_Noisey);    % Test resulting in High Confidence - No confidence noise
            [hmmm3 bop3] = intersect(hmmm1,hmmm2);                  % Tests BOTH categorised LOW & High Confidence
            Results_Array_Confidence_Low_Category_Noisey(row,1)=Tested_Vals(row);
            if size(hmmm1,1) == 0
                % No LOW categorisations - so set to full number of tests
                Results_Array_Confidence_Low_Category_Noisey(row,2)=size(idxs,1) ;
            elseif size(hmmm3,1) == 0
                % No LOW categorisations w/ High Confidence - so set to full number of tests
                Results_Array_Confidence_Low_Category_Noisey(row,2)=size(idxs,1) ;
            else
                Results_Array_Confidence_Low_Category_Noisey(row,2)=size(idxs,1) - sum(Exp_Results(hmmm3,4));
            end
            Results_Array_Confidence_Low_Category_Noisey(row,3)=size(idxs,1);

            [hmmm1 bop1] = intersect(idxs,idxs_High_Cat);
            [hmmm2 bop2] = intersect(idxs,idxs_High_Conf_Clean);
            [hmmm3 bop3] = intersect(hmmm1,hmmm2);
            Results_Array_Confidence_High_Category_Clean(row,1)=Tested_Vals(row);
            if size(hmmm3,1) == 0
                Results_Array_Confidence_High_Category_Clean(row,2)=0;
            else
                Results_Array_Confidence_High_Category_Clean(row,2)=sum(Exp_Results(hmmm3,3));
            end
            Results_Array_Confidence_High_Category_Clean(row,3)=size(idxs,1);

            [hmmm2 bop2] = intersect(idxs,idxs_High_Conf_Noisey);
            [hmmm3 bop3] = intersect(hmmm1,hmmm2);
            Results_Array_Confidence_High_Category_Noisey(row,1)=Tested_Vals(row);
            if size(hmmm3,1) == 0
                Results_Array_Confidence_High_Category_Noisey(row,2)=0;
            else
                Results_Array_Confidence_High_Category_Noisey(row,2)=sum(Exp_Results(hmmm3,4));
            end
            Results_Array_Confidence_High_Category_Noisey(row,3)=size(idxs,1);

        end

        options.expType = "YesNo"        % Specify the design of the 'experiment' - a forced choice categorisation task 

        % Fit a cumulative Gaussian psychmetric curve to categorisation data
        Psychometric = psignifit(Results_Array_Perception,options);
        figure(1)
        h1 = plotPsych(Psychometric,plotOptions);
        PSE_Perception = Psychometric.Fit(1);
        [val PSE_idx]= find(h1.XData >= PSE_Perception,1);
        PSE_Pr = h1.YData(PSE_idx);
        Slopes(Crit_N,1) = getSlopePC(Psychometric,PSE_Pr);
%        Slopes(Crit_N,1) = getThreshold(Psychometric,0.75) - getThreshold(Psychometric,0.25);

        Psychometric2 = psignifit(Results_Array_Confidence_Low_Category_Clean,options);
        figure(2)
        clf
        h2 = plotPsych(Psychometric2,plotOptions);
        PSE = Psychometric2.Fit(1);
        [val PSE_idx]= find(h2.XData >= PSE,1);
        PSE_Pr = h2.YData(PSE_idx);
        Lower_Cat_Confidence_Slope = getSlopePC(Psychometric2,PSE_Pr);    
%        Lower_Cat_Confidence_Slope = getThreshold(Psychometric2,0.75) - getThreshold(Psychometric2,0.25);


        Psychometric3 = psignifit(Results_Array_Confidence_High_Category_Clean,options);
        figure(2)
        hold on
        h3 = plotPsych(Psychometric3,plotOptions);
        PSE = Psychometric3.Fit(1);
        [val PSE_idx]= find(h3.XData >= PSE,1);
        PSE_Pr = h3.YData(PSE_idx);
        Upper_Cat_Confidence_Slope = getSlopePC(Psychometric3,PSE_Pr);    
%        Upper_Cat_Confidence_Slope = getThreshold(Psychometric3,0.75) - getThreshold(Psychometric3,0.25);

        Candidate_Slopes = abs(([Slopes(Crit_N,1)-Lower_Cat_Confidence_Slope Slopes(Crit_N,1)-Upper_Cat_Confidence_Slope]));
        [Biggest_Diff idx] = max(Candidate_Slopes);
        if idx == 1
            Slopes(Crit_N,2) = Lower_Cat_Confidence_Slope;
        else
            Slopes(Crit_N,2) = Upper_Cat_Confidence_Slope;
        end

        Psychometric2 = psignifit(Results_Array_Confidence_Low_Category_Noisey,options);
        figure(3)
        clf
        h2 = plotPsych(Psychometric2,plotOptions);
        PSE = Psychometric2.Fit(1);
        [val PSE_idx]= find(h2.XData >= PSE,1);
        PSE_Pr = h2.YData(PSE_idx);
        Lower_Cat_Confidence_Slope = getSlopePC(Psychometric2,PSE_Pr);    
%        Lower_Cat_Confidence_Slope = getThreshold(Psychometric2,0.75) - getThreshold(Psychometric2,0.25);


        Psychometric3 = psignifit(Results_Array_Confidence_High_Category_Noisey,options);
        figure(3)
        hold on
        h3 = plotPsych(Psychometric3,plotOptions);
        PSE = Psychometric3.Fit(1);
        [val PSE_idx]= find(h3.XData >= PSE,1);
        PSE_Pr = h3.YData(PSE_idx);
        Upper_Cat_Confidence_Slope = getSlopePC(Psychometric3,PSE_Pr);    
%        Upper_Cat_Confidence_Slope = getThreshold(Psychometric3,0.75) - getThreshold(Psychometric3,0.25);

        Candidate_Slopes = abs(([Slopes(Crit_N,1)-Lower_Cat_Confidence_Slope Slopes(Crit_N,1)-Upper_Cat_Confidence_Slope]));
        [Biggest_Diff idx] = max(Candidate_Slopes);
        if idx == 1
            Slopes(Crit_N,3) = Lower_Cat_Confidence_Slope;
        else
            Slopes(Crit_N,3) = Upper_Cat_Confidence_Slope;
        end
        

        %%

        % Create pseudo random distributions of Signal and Noise values
        Noise_Dist = pearsrnd(-Ch_SDs*0.5,Ch_SDs,+Skew,Kurtosis,1,N_Simulated_Trials/2);
        Signal_Dist = pearsrnd(+Ch_SDs*0.5,Ch_SDs,-Skew,Kurtosis,1,N_Simulated_Trials/2);

        % Convert Signal and Noise values into (unsigned) absolute values for
        % confidence classifications
        Noise_Confidence_Responses = abs(Noise_Dist);
        Signal_Confidence_Responses = abs(Signal_Dist);

        % Find Noise values to classify as having elicited high- or
        % low-confidence
        [vals idxs_Noise_High_Confidence] = find(Noise_Confidence_Responses >= Confidence_Criteria(Crit_N));
        [vals idxs_Noise_Low_Confidence] = find(Noise_Confidence_Responses < Confidence_Criteria(Crit_N));
        % Find Noise values to categorise as a Noise (values below category 
        % criterion) or signal (values equal to or above category criterion)
        [vals idxs_Noise_Cat1] = find(Noise_Dist < Category_Criterion);
        [vals idxs_Noise_Cat2] = find(Noise_Dist >= Category_Criterion);

        % Find Noise values classified as Noise with high-confidence
        [idxs_High_Conf_Noise_N vals]=intersect(idxs_Noise_High_Confidence,idxs_Noise_Cat1);
        % Find Noise values classified as Noise with low-confidence
        [idxs_Low_Conf_Noise_N vals]=intersect(idxs_Noise_Low_Confidence,idxs_Noise_Cat1); 
        % Find Noise values classified as Signal with low-confidence
        [idxs_Low_Conf_Signal_N vals]=intersect(idxs_Noise_Low_Confidence,idxs_Noise_Cat2); 
        % Find Noise values classified as Signal with high-confidence
        [idxs_High_Conf_Signal_N vals]=intersect(idxs_Noise_High_Confidence,idxs_Noise_Cat2); 


        % Find Signal values to classify as having elicited high- or
        % low-confidence
        [vals idxs_Signal_High_Confidence] = find(Signal_Confidence_Responses >= Confidence_Criteria(Crit_N));
        [vals idxs_Signal_Low_Confidence] = find(Signal_Confidence_Responses < Confidence_Criteria(Crit_N));
        % Find Signal values to categorise as a Noise (values below category 
        % criterion) or signal (values equal to or above category criterion)
        [vals idxs_Signal_Cat1] = find(Signal_Dist < Category_Criterion);
        [vals idxs_Signal_Cat2] = find(Signal_Dist >= Category_Criterion);

        % Find Signal values classified as Noise with high-confidence
        [idxs_High_Conf_Noise_S vals]=intersect(idxs_Signal_High_Confidence,idxs_Signal_Cat1);
        % Find Signal values classified as Noise with low-confidence
        [idxs_Low_Conf_Noise_S vals]=intersect(idxs_Signal_Low_Confidence,idxs_Signal_Cat1); 
        % Find Signal values classified as Signal with low-confidence
        [idxs_Low_Conf_Signal_S vals]=intersect(idxs_Signal_Low_Confidence,idxs_Signal_Cat2); 
        % Find Signal values classified as Signal with high-confidence
        [idxs_High_Conf_Signal_S vals]=intersect(idxs_Signal_High_Confidence,idxs_Signal_Cat2); 

        % Create meta-d array, and fill the first row with Noise
        % classifications X Confidence
        Meta_D_Array=[];
        Meta_D_Array(1,1) = size(idxs_High_Conf_Noise_N,2);
        Meta_D_Array(1,2) = size(idxs_Low_Conf_Noise_N,2);
        Meta_D_Array(1,3) = size(idxs_Low_Conf_Signal_N,2);
        Meta_D_Array(1,4) = size(idxs_High_Conf_Signal_N,2);

        % Fill second row with Signal classifications X Confidence
        Meta_D_Array(2,1) = size(idxs_High_Conf_Noise_S,2);
        Meta_D_Array(2,2) = size(idxs_Low_Conf_Noise_S,2);
        Meta_D_Array(2,3) = size(idxs_Low_Conf_Signal_S,2);
        Meta_D_Array(2,4) = size(idxs_High_Conf_Signal_S,2);

        if min(min(Meta_D_Array)) > 0
            % Conduct SDT-based analysis of confidence
            Analysis = fit_meta_d_MLE(Meta_D_Array(1,:),Meta_D_Array(2,:));
            % Store estimated d' values for excess kurtosis simmulations
            dPrimes(Crit_N)=Analysis.d1;
            % Store estimated meta d' values for excess kurtosis simmulations
            Meta_dPrimes(Crit_N)=Analysis.meta_d            
        else
            % Store estimated d' values for excess kurtosis simmulations
            dPrimes(1,Crit_N)=NaN;
            % Store estimated meta d' values for excess kurtosis simmulations
            Meta_dPrimes(1,Crit_N)=NaN;
        end        

        %%
        idxs_Noisey_High_Confidence=[];
        idxs_Noisey_Low_Confidence=[];
        idxs_Signaly_High_Confidence=[];
        idxs_Signaly_Low_Confidence=[];

        Noisey_Confidence_Crit = (Confidence_Criteria(Crit_N)-(Confidence_Criteria(Crit_N)*Confidence_Noise)) + rand(2,size(Noise_Dist,2))*(Confidence_Criteria(Crit_N)*Confidence_Noise*2);
        for t = 1 : size(Noisey_Confidence_Crit,2)

            if Noise_Confidence_Responses(t) >= Noisey_Confidence_Crit(1,t)
                idxs_Noisey_High_Confidence(end+1) = t;
            else
                idxs_Noisey_Low_Confidence(end+1) = t;
            end

            if Signal_Confidence_Responses(t) >= Noisey_Confidence_Crit(2,t)
                idxs_Signaly_High_Confidence(end+1) = t;
            else
                idxs_Signaly_Low_Confidence(end+1) = t;
            end

        end
        % Find Noise values classified as Noise with high-confidence
        [idxs_High_Conf_Noisey_N vals]=intersect(idxs_Noisey_High_Confidence,idxs_Noise_Cat1);
        % Find Noise values classified as Noise with low-confidence
        [idxs_Low_Conf_Noisey_N vals]=intersect(idxs_Noisey_Low_Confidence,idxs_Noise_Cat1); 
        % Find Noise values classified as Signal with low-confidence
        [idxs_Low_Conf_Signaly_N vals]=intersect(idxs_Noisey_Low_Confidence,idxs_Noise_Cat2); 
        % Find Noise values classified as Signal with high-confidence
        [idxs_High_Conf_Signaly_N vals]=intersect(idxs_Noisey_High_Confidence,idxs_Noise_Cat2); 


        % Find Signal values classified as Noise with high-confidence
        [idxs_High_Conf_Noisey_S vals]=intersect(idxs_Signaly_High_Confidence,idxs_Signal_Cat1);
        % Find Signal values classified as Noise with low-confidence
        [idxs_Low_Conf_Noisey_S vals]=intersect(idxs_Signaly_Low_Confidence,idxs_Signal_Cat1); 
        % Find Signal values classified as Signal with low-confidence
        [idxs_Low_Conf_Signaly_S vals]=intersect(idxs_Signaly_Low_Confidence,idxs_Signal_Cat2); 
        % Find Signal values classified as Signal with high-confidence
        [idxs_High_Conf_Signaly_S vals]=intersect(idxs_Signaly_High_Confidence,idxs_Signal_Cat2); 

        % Create meta-d array, and fill the first row with Noise
        % classifications X Confidence
        Meta_D_Array=[];
        Meta_D_Array(1,1) = size(idxs_High_Conf_Noisey_N,2);
        Meta_D_Array(1,2) = size(idxs_Low_Conf_Noisey_N,2);
        Meta_D_Array(1,3) = size(idxs_Low_Conf_Signaly_N,2);
        Meta_D_Array(1,4) = size(idxs_High_Conf_Signaly_N,2);

        % Fill second row with Signal classifications X Confidence
        Meta_D_Array(2,1) = size(idxs_High_Conf_Noisey_S,2);
        Meta_D_Array(2,2) = size(idxs_Low_Conf_Noisey_S,2);
        Meta_D_Array(2,3) = size(idxs_Low_Conf_Signaly_S,2);
        Meta_D_Array(2,4) = size(idxs_High_Conf_Signaly_S,2);

        if min(min(Meta_D_Array)) > 0
            nRatings1 = Meta_D_Array(1,:);
            nRatings2 = Meta_D_Array(2,:);
            
            % Conduct SDT-based analysis of confidence - Using Maniscarlo & Lau's code
            Analysis = fit_meta_d_MLE(Meta_D_Array(1,:),Meta_D_Array(2,:));
            % Store estimated d' values for excess kurtosis simmulations
            Noisey_dPrimes(Crit_N)=Analysis.d1;
            % Store estimated meta d' values for excess kurtosis simmulations
            Noisey_Meta_dPrimes(Crit_N)=Analysis.meta_d
        else
            % Store estimated d' values for excess kurtosis simmulations
            Noisey_dPrimes(Crit_N)=NaN;
            % Store estimated meta d' values for excess kurtosis simmulations
            Noisey_Meta_dPrimes(Crit_N)=NaN;
        end


    end

    Slope_Ratios_Clean = Slopes(:,2)./Slopes(:,1);
    Slope_Ratios_Noisey = Slopes(:,3)./Slopes(:,1);

    Meta_d_Ratios = Meta_dPrimes./dPrimes;
    Grey_Metas = isnan(Meta_d_Ratios);
    Grey_Metas = double(Grey_Metas);
    Grey_Metas(Grey_Metas==0)=NaN;

    Noisey_Meta_d_Ratios = Noisey_Meta_dPrimes./Noisey_dPrimes;
    Grey_Noisey_Metas = isnan(Noisey_Meta_d_Ratios);
    Grey_Noisey_Metas = double(Grey_Noisey_Metas);
    Grey_Noisey_Metas(Grey_Noisey_Metas==0)=NaN;

    The_Mins = [min(Slope_Ratios_Clean) min(Slope_Ratios_Noisey) min(Meta_d_Ratios) min(Noisey_Meta_d_Ratios)];
    The_Min = min(The_Mins);

    The_Maxs = [max(Slope_Ratios_Clean) max(Slope_Ratios_Noisey) max(Meta_d_Ratios) max(Noisey_Meta_d_Ratios)];
    The_Max = max(The_Maxs);

    figure(4)
%     if Skew == 0
%         sgtitle(strcat('Skew Difference = 0.0',num2str(round(-Skew*2,2))))
%     else
%         sgtitle(strcat('Skew Difference = ',num2str(round(-Skew*2,2))))
%     end

    subplot(2,2,1)
    hold on
    if N_Skew == 1
        h=fill([Confidence_Criteria(1),Confidence_Criteria(1),max(Confidence_Criteria),max(Confidence_Criteria)],[0.95,1.05,1.05,0.95],'g','LineStyle','none');
        h.FaceAlpha=0.25;
        plot([Confidence_Criteria(1),max(Confidence_Criteria)],[1,1],'-g','LineWidth',3)
        scatter(Confidence_Criteria,Slope_Ratios_Clean,'k','filled')
    elseif N_Skew == 2
        scatter(Confidence_Criteria,Slope_Ratios_Clean,'b','filled')
    else
        scatter(Confidence_Criteria,Slope_Ratios_Clean,'r','filled')
    end
    xlabel('Confidence Criteria (degs)','FontSize',16,'FontWeight','bold')  
    ylabel('Slope Ratios','FontSize',16,'FontWeight','bold')  
    box off
    ax=gca;
    ax.FontSize = 14; 
    ylim([The_Min The_Max])
    xlim([Confidence_Criteria(1) max(Confidence_Criteria)])
    title({'Slope Tests' 'No Confidence Noise'},'FontSize',16,'FontWeight','bold')


    subplot(2,2,2)
    hold on
    if N_Skew == 1
        plot([Confidence_Criteria(1),max(Confidence_Criteria)],[1,1],'-g','LineWidth',5)
        h=fill([Confidence_Criteria(1),Confidence_Criteria(1),max(Confidence_Criteria),max(Confidence_Criteria)],[0.95,1.05,1.05,0.95],'g','LineStyle','none');
        h.FaceAlpha=0.25;
        plot([Confidence_Criteria(1),max(Confidence_Criteria)],[1,1],'-g','LineWidth',3)
        scatter(Confidence_Criteria,Slope_Ratios_Noisey,'k','filled')
    elseif N_Skew == 2
        scatter(Confidence_Criteria,Slope_Ratios_Noisey,'b','filled')
    else
        scatter(Confidence_Criteria,Slope_Ratios_Noisey,'r','filled')
    end
    xlabel('Confidence Criteria (degs)','FontSize',16,'FontWeight','bold')  
    ylabel('Slope Ratios','FontSize',16,'FontWeight','bold')  
    box off
    ax=gca;
    ax.FontSize = 14; 
    ylim([The_Min The_Max])
    xlim([Confidence_Criteria(1) max(Confidence_Criteria)])
    title({'Slope Tests' '75% Confidence Noise'},'FontSize',16,'FontWeight','bold')

    subplot(2,2,3)
    hold on
    if N_Skew == 1
        h=fill([Confidence_Criteria(1),Confidence_Criteria(1),max(Confidence_Criteria),max(Confidence_Criteria)],[0.95,1.05,1.05,0.95],'g','LineStyle','none');
        h.FaceAlpha=0.25;
        plot([Confidence_Criteria(1),max(Confidence_Criteria)],[1,1],'-g','LineWidth',3)
        scatter(Confidence_Criteria,Meta_d_Ratios,'k','filled')
    elseif N_Skew == 2
        scatter(Confidence_Criteria,Meta_d_Ratios,'b','filled')
    else
        scatter(Confidence_Criteria,Meta_d_Ratios,'r','filled')
    end
    scatter(Confidence_Criteria,Grey_Metas,'c','filled')
    xlabel('Confidence Criteria (degs)','FontSize',16,'FontWeight','bold')  
    ylabel('Meta dPrime : dPrim Ratios','FontSize',16,'FontWeight','bold')  
    box off
    ax=gca;
    ax.FontSize = 14; 
    ylim([The_Min The_Max])
    xlim([Confidence_Criteria(1) max(Confidence_Criteria)])
    title({'meta dPrime : dPrime' 'No Confidence Noise'},'FontSize',16,'FontWeight','bold')

    subplot(2,2,4)
    hold on
    if N_Skew == 1
        plot([Confidence_Criteria(1),max(Confidence_Criteria)],[1,1],'-g','LineWidth',5)
        h=fill([Confidence_Criteria(1),Confidence_Criteria(1),max(Confidence_Criteria),max(Confidence_Criteria)],[0.95,1.05,1.05,0.95],'g','LineStyle','none');
        h.FaceAlpha=0.25;
        plot([Confidence_Criteria(1),max(Confidence_Criteria)],[1,1],'-g','LineWidth',3)
        scatter(Confidence_Criteria,Noisey_Meta_d_Ratios,'k','filled')
    elseif N_Skew == 2
        scatter(Confidence_Criteria,Noisey_Meta_d_Ratios,'b','filled')
    else
        scatter(Confidence_Criteria,Noisey_Meta_d_Ratios,'r','filled')
    end
    scatter(Confidence_Criteria,Grey_Noisey_Metas,'c','filled')
    xlabel('Confidence Criteria (degs)','FontSize',16,'FontWeight','bold')  
    ylabel('Meta dPrime : dPrime Ratios','FontSize',16,'FontWeight','bold')  
    box off
    ax=gca;
    ax.FontSize = 14; 
    ylim([The_Min The_Max])
    xlim([Confidence_Criteria(1) max(Confidence_Criteria)])
    title({'Meta dPrime : dPrime' '75% Confidence Noise'},'FontSize',16,'FontWeight','bold')
end
